﻿using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows.Forms;
using LC;

namespace LC
{
    public class GPSEventArgs : EventArgs
    {
        #region 纬度
        private int m_LatitudeDegree = 0;
        /// <summary>
        /// 纬度(度)
        /// </summary>
        public int LatitudeDegree
        {
            get { return m_LatitudeDegree; }
        }

        private double m_LatitudeMinute = 0.00;
        /// <summary>
        /// 纬度(分)
        /// </summary>
        public double LatitudeMinute
        {
            get { return m_LatitudeMinute; }
        }
        #endregion

        #region 经度
        private int m_LongitudeDegree = 0;
        /// <summary>
        /// 经度
        /// </summary>
        public int LongitudeDegree
        {
            get { return m_LongitudeDegree; }
        }

        private double m_LongitudeMinute = 0.00;
        /// <summary>
        /// 经度
        /// </summary>
        public double LongitudeMinute
        {
            get { return m_LongitudeMinute; }
        }
        #endregion

        #region 速度
        private double m_speed = 0.00;
        /// <summary>
        /// 速度
        /// </summary>
        public double Speed
        {
            get { return m_speed; }
        }
        #endregion

        #region 航向(方位角)
        private double m_bearing = 0.00;
        /// <summary>
        /// 航向(方位角)
        /// </summary>
        public double Bearing
        {
            get { return m_bearing; }
        }
        #endregion

        #region 定位状态
        private bool m_Status = false;
        /// <summary>
        /// 定位状态
        /// </summary>
        public bool Status
        {
            get { return m_Status; }
        }
        #endregion

        #region 最后一次定位时间
        private DateTime lastestLocateTime = DateTime.Now;
        /// <summary>
        /// 最后一次定位时间
        /// </summary>
        public DateTime LastestLocateTime
        {
            get { return lastestLocateTime; }
            set { lastestLocateTime = value; }
        }
        #endregion

        #region GPS定位状态
        private int m_locationStatus = 0;
        /// <summary>
        /// GPS定位状态
        /// </summary>
        public int LocationStatus
        {
            get { return m_locationStatus; }
        }
        #endregion

        #region 当前卫星总数
        private int m_SatelliteTotalNums = 0;
        /// <summary>
        /// 当前卫星总数
        /// </summary>
        public int SatelliteTotalNums
        {
            get { return m_SatelliteTotalNums; }
        }
        #endregion

        #region 卫星信息
        private List<SatellitesInfo> m_satellitesInfoList = new List<SatellitesInfo>();
        /// <summary>
        /// 卫星信息
        /// </summary>
        public List<SatellitesInfo> SatellitesInfoList
        {
            get { return m_satellitesInfoList; }
        }
        #endregion

        public GPSEventArgs(bool status, int longtitudeDegree, double longtitudeMinute, int latitudeDegree, double latitudeMinute, double speed, double bearing, int locationStatus, int satelliteCount, List<SatellitesInfo> list)
        {
            this.m_Status = status;
            this.m_LongitudeDegree = longtitudeDegree; 
            this.m_LongitudeMinute = longtitudeMinute;
            this.m_LatitudeDegree = latitudeDegree;
            this.m_LatitudeMinute = latitudeMinute;
            this.m_speed = speed;
            this.m_bearing = bearing;
            this.m_locationStatus = locationStatus;
            this.m_SatelliteTotalNums = satelliteCount;
            this.m_satellitesInfoList = list;
        }
    }

    public delegate void GetGPSInfoHandler(object sender,GPSEventArgs e);

    public class GpsManager
    {
        private NLog.Logger logger = NLog.LogManager.GetLogger("GpsManager");
        private bool IsFirstSend = true;
        private uint CurrentTickCount = 0;

        #region 是否启动
        private bool m_IsStart = false;
        /// <summary>
        /// 是否启动
        /// </summary>
        public bool IsStart
        {
            get { return m_IsStart; }
            set { m_IsStart = value; }
        }
        #endregion

        #region GPS信息
        #region 发送时间间隔
        private uint m_SendTimeSpan = 10;
        /// <summary>
        /// 发送时间间隔
        /// </summary>
        public uint SendTimeSpan
        {
            get { return m_SendTimeSpan * 1000; }
            set { m_SendTimeSpan = value; }
        }
        #endregion

        #region 上一次发送时间
        private uint m_LastSendTime = 0;
        /// <summary>
        /// 上一次发送时间
        /// </summary>
        public uint LastSendTime
        {
            get { return m_LastSendTime; }
            set { m_LastSendTime = value; }
        }
        #endregion

        #region 发送GPS数据事件
        /// <summary>
        /// 发送GPS数据
        /// </summary>
        public event GetGPSInfoHandler OnGetGPSInfo = null;
        #endregion

        #region GPS定位状态
        private byte m_locationStatus = 0;
        /// <summary>
        /// GPS状态(0=未定位, 1=非差分定位, 2=差分定位, 3=无效PPS, 6=正在估算)
        /// </summary>
        public byte LocationStatus
        {
            get { return m_locationStatus; }
        }
        #endregion

        #region 定位状态
        private bool m_Status = false;
        /// <summary>
        /// 定位状态
        /// </summary>
        public bool Status
        {
            get { return m_Status; }
        }
        #endregion

        #region 纬度
        private int m_LatitudeDegree = 0;
        /// <summary>
        /// 纬度(度)
        /// </summary>
        public int LatitudeDegree
        {
            get { return m_LatitudeDegree; }
        }

        private double m_LatitudeMinute = 0;
        /// <summary>
        /// 纬度(分)
        /// </summary>
        public double LatitudeMinute
        {
            get { return m_LatitudeMinute; }
        }
        #endregion

        #region 经度
        private int m_LongitudeDegree = 0;
        /// <summary>
        /// 经度(度)
        /// </summary>
        public int LongitudeDegree
        {
            get { return m_LongitudeDegree; }
        }

        private double m_LongitudeMinute = 0.00;
        /// <summary>
        /// 经度(分)
        /// </summary>
        public double LongitudeMinute
        {
            get { return m_LongitudeMinute; }
        }
        #endregion

        #region 速度
        private byte m_speed = 0;
        /// <summary>
        /// 速度
        /// </summary>
        public byte Speed
        {
            get { return m_speed; }
        }
        #endregion

        #region 定位时间
        public DateTime SatelliteTime = DateTime.Now;
        #endregion

        #region 航向(方位角)
        private short m_bearing = 0;
        /// <summary>
        /// 航向(方位角)
        /// </summary>
        public short Bearing
        {
            get { return m_bearing; }
        }
        #endregion

        #region 记录时间
        private DateTime recTime;
        /// <summary>
        /// 记录时间
        /// </summary>
        public DateTime RecTime
        {
            get { return recTime; }
        }
        #endregion

        #region 当前卫星总数
        private byte m_SatelliteTotalNums = 0;
        /// <summary>
        /// 当前卫星总数
        /// </summary>
        public byte SatelliteTotalNums
        {
            get { return m_SatelliteTotalNums; }
        }
        #endregion

        #region 卫星信号列表
        private List<SatellitesInfo> m_satellitesInfoList = new List<SatellitesInfo>();
        /// <summary>
        /// 卫星信号列表
        /// </summary>
        public List<SatellitesInfo> SatellitesInfoList
        {
            get { return m_satellitesInfoList; }
            set { m_satellitesInfoList = value; }
        }
        #endregion
        #endregion

        #region NmeaInterpreter
        NmeaInterpreter nameInterpreter = null;

        private double m_MaximumDop = 6.0;
        /// <summary>
        /// 最大精度衰减因子
        /// </summary>
        public double MaximumDOP
        {
            get { return m_MaximumDop; }
            set
            {
                m_MaximumDop = value;
                m_MaximumErrorInMeters = m_MaximumDop * 6.0;
            }
        }

        private double m_MaximumErrorInMeters = 36;
        /// <summary>
        /// 最大误差距离
        /// </summary>
        public double MaximumError
        {
            get { return m_MaximumErrorInMeters; }
            set
            {
                m_MaximumErrorInMeters = value;
                m_MaximumDop = m_MaximumErrorInMeters / 6.0;
            }
        }

        /// <summary>
        /// 当前水平坐标的精密值强弱度
        /// </summary>
        private double CurrentHDOP;
        #endregion

        #region 串口定义
        /// <summary>
        /// 串口
        /// </summary>
        private SerialPort serialPort = null;

        private string portName = "COM7";
        /// <summary>
        /// 串口名称
        /// </summary>
        public string CommPortName
        {
            get { return portName; }
            set { portName = value; }
        }

        private int baudRate = 4800;
        /// <summary>
        /// 波特率
        /// </summary>
        public int BaudRate
        {
            get { return baudRate; }
            set { baudRate = value; }
        }

        private Parity parity = Parity.None;
        /// <summary>
        /// 奇偶检验
        /// </summary>
        public Parity Parity
        {
            get { return parity; }
            set { parity = value; }
        }

        private int dataBits = 8;
        /// <summary>
        /// 数据位数
        /// </summary>
        public int DataBits
        {
            get { return dataBits; }
            set { dataBits = value; }
        }

        private StopBits stopBits = StopBits.One;
        /// <summary>
        /// 停止位
        /// </summary>
        public StopBits StopBits
        {
            get { return stopBits; }
            set { stopBits = value; }
        }
        #endregion

        public GpsManager()
        {
            
        }

        ~GpsManager()
        {
            if (serialPort != null && serialPort.IsOpen)
            {
                serialPort.Close();
            }
            if (serialPort != null)
            {
                serialPort.Dispose();
                serialPort = null;
            }
        }

        /// <summary>
        /// 开始读取GPS模块数据
        /// </summary>
        public bool Start()
        {
            bool bResult = true;
            try
            {
                nameInterpreter = new NmeaInterpreter();
                nameInterpreter.PositionReceived += new NmeaInterpreter.PositionReceivedEventHandler(OnPositionReceived);
                nameInterpreter.HDOPReceived += new NmeaInterpreter.HDOPReceivedEventHandler(OnHDOPReceived);
                nameInterpreter.SatelliteReceived += new NmeaInterpreter.SatelliteReceivedEventHandler(OnSatelliteReceived);
                nameInterpreter.FixObtained += new NmeaInterpreter.FixObtainedEventHandler(OnFixObtained);
                nameInterpreter.FixLost += new NmeaInterpreter.FixLostEventHandler(OnFixLost);
                nameInterpreter.LocationStatusReceived += new NmeaInterpreter.LocationStatusReceivedEventHandler(OnLocationStatusReceived);
                nameInterpreter.SpeedReceived += new NmeaInterpreter.SpeedReceivedEventHandler(OnSpeedReceived);
                nameInterpreter.BearingReceived += new NmeaInterpreter.BearingReceivedEventHandler(OnBearingReceived);
                nameInterpreter.DateTimeChanged += new NmeaInterpreter.DateTimeChangedEventHandler(OnSatelliteTimeChanged);

                serialPort = new SerialPort(portName, baudRate, parity, dataBits, stopBits);
                serialPort.Open();

                if (serialPort != null && serialPort.IsOpen)
                {
                    if (Device.GetGpsPowerStatus() != 1)
                    {
                        Device.EnableGpsModule();
                    }
                    Thread.Sleep(300);
                    bool powerOnFlag = Device.GetGpsPowerStatus() == 1 ? true : false;

                    if (!powerOnFlag)
                    {
                        bResult = false;
                        logger.Error("Power on fail!");
                    }
                    Thread.Sleep(300);
                }
                else
                {
                    bResult = false;
                }

                if (bResult && serialPort.IsOpen)
                {
                    logger.Info("serialPort.DataReceived注册");
                    serialPort.DataReceived += new SerialDataReceivedEventHandler(OnDataReceived);
                    m_IsStart = true;
                }
            }
            catch (Exception ex)
            {
                logger.ErrorException("Open serialport exception", ex);
                bResult = false;
            }

            return bResult;
        }

        /// <summary>
        /// 停止读取GPS模块数据
        /// </summary>
        public void Stop()
        {
            try
            {
                nameInterpreter = null;
                if (serialPort != null && serialPort.IsOpen)
                {
                    serialPort.DataReceived -= new SerialDataReceivedEventHandler(OnDataReceived);
                    serialPort.DiscardOutBuffer();
                    serialPort.Close();
                    serialPort.Dispose();
                    serialPort = null;
                }

                Device.DisableGpsModule();
                Thread.Sleep(300);
                m_IsStart = false;
            }
            catch (Exception ex)
            {
                logger.ErrorException("Close serialport exception", ex);
            }
            logger.Info("Stop GPS Reader");
        }

        /// <summary>
        /// 清除GPS数据
        /// </summary>
        private void Clear()
        {
            m_locationStatus = 0;
            m_LongitudeDegree = 0;
            m_LongitudeMinute = 0.00;
            m_LatitudeDegree = 0;
            m_LatitudeMinute = 0.00;
            m_speed = 0;
            m_bearing = 0;
            m_SatelliteTotalNums = 0;
            m_satellitesInfoList = new List<SatellitesInfo>();
        }

        private void OnDataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                while (serialPort != null && serialPort.BytesToRead > 0)
                {
                    Application.DoEvents();
                    OnHandleData(serialPort.ReadLine());
                }
            }
            catch (Exception ex)
            {
                logger.Error("read data exception,detail:{0}{1}", ex.Message, ex.StackTrace);
            }
        }

        private void OnHandleData(string rawData)
        {
            //logger.Debug(rawData);
            if (String.IsNullOrEmpty(rawData))
            {
                return;
            }
            if (!nameInterpreter.Parse(rawData.Trim()))
            {
                logger.Error("An NMEA sentence was not recognized");
            }
        }

        private void OnHDOPReceived(double value)
        {
            // Remember the current HDOP value
            CurrentHDOP = value;
        }

        private void OnPositionReceived(string latitudeDegree, string latitudeMinute, string NorS, string longitudeDegree, string longitudeMinute, string EorW)
        {
            //logger.Info(String.Format("Current Postion(LatitudeDegree:{0},LatitudeMinute:{1},LongitudeDegree:{2},LongitudeMinute:{3}),Status:{4},CurrentHDOP:{5},MaximumDOP:{6}", latitudeDegree, latitudeMinute, longitudeDegree, longitudeMinute, Status.ToString(), CurrentHDOP, MaximumDOP));

            // Is the HDOP at least six?
            if (Status && SatelliteTotalNums >= 3 && CurrentHDOP <= MaximumDOP)
            {
                // Yes. Display the current position
                recTime = DateTime.Now;
                this.m_LatitudeDegree = int.Parse(latitudeDegree) * (NorS == "N" ? 1 : -1);
                this.m_LatitudeMinute = double.Parse(latitudeMinute) * (NorS == "N" ? 1 : -1);
                this.m_LongitudeDegree = int.Parse(longitudeDegree) * (EorW == "E" ? 1 : -1);
                this.m_LongitudeMinute = double.Parse(longitudeMinute) * (EorW == "E" ? 1 : -1);
            }

            CurrentTickCount = Win32.GetTickCount();
            if (Status)
            {
                logger.Debug("已定位，发送GPS信息");
                //已定位，发送GPS信息
                if (m_LatitudeDegree > 0 && m_LatitudeMinute > 0 && m_LongitudeDegree > 0 && m_LongitudeMinute > 0)
                {
                    if (IsFirstSend)
                    {
                        GPSEventArgs e = new GPSEventArgs(this.m_Status, this.m_LongitudeDegree, this.m_LongitudeMinute, this.m_LatitudeDegree, this.m_LatitudeMinute, this.m_speed, this.m_bearing, this.m_locationStatus, this.m_SatelliteTotalNums, this.m_satellitesInfoList);
                        if (OnGetGPSInfo != null)
                            OnGetGPSInfo(this, e);

                        IsFirstSend = false;
                        this.m_LastSendTime = Win32.GetTickCount();
                    }
                    else
                    {
                        if ((CurrentTickCount - this.m_LastSendTime) > SendTimeSpan)
                        {
                            GPSEventArgs e = new GPSEventArgs(this.m_Status, this.m_LongitudeDegree, this.m_LongitudeMinute, this.m_LatitudeDegree, this.m_LatitudeMinute, this.m_speed, this.m_bearing, this.m_locationStatus, this.m_SatelliteTotalNums, this.m_satellitesInfoList);
                            if (OnGetGPSInfo != null)
                                OnGetGPSInfo(this, e);

                            this.m_LastSendTime = Win32.GetTickCount();
                        }
                    }
                }
            }
        }

        private void OnSatelliteReceived(byte satelliteTotalNums, byte pseudoRandomCode, short azimuth, byte elevation, byte signalToNoiseRatio)
        {
            this.m_SatelliteTotalNums = satelliteTotalNums;
            SatellitesInfoList.Add(new SatellitesInfo(pseudoRandomCode, azimuth, elevation, signalToNoiseRatio));
        }

        private void OnFixObtained()
        {
            if (!this.m_Status)
            {
                this.m_Status = true;
                GPSEventArgs e = new GPSEventArgs(this.m_Status, this.m_LongitudeDegree, this.m_LongitudeMinute, this.m_LatitudeDegree, this.m_LatitudeMinute, this.m_speed, this.m_bearing, this.m_locationStatus, this.m_SatelliteTotalNums, this.m_satellitesInfoList);

                if (OnGetGPSInfo != null)
                    OnGetGPSInfo(this, e);
            }
        }

        private void OnFixLost()
        {
            if (this.m_Status)
            {
                this.m_Status = false;
                GPSEventArgs e = new GPSEventArgs(this.m_Status, this.m_LongitudeDegree, this.m_LongitudeMinute, this.m_LatitudeDegree, this.m_LatitudeMinute, this.m_speed, this.m_bearing, this.m_locationStatus, this.m_SatelliteTotalNums, this.m_satellitesInfoList);

                if (OnGetGPSInfo != null)
                    OnGetGPSInfo(this, e);
            }
        }

        private void OnLocationStatusReceived(byte status)
        {
            m_locationStatus = status;
            if (status == 1 || status == 2)
            {
                if (!m_Status)
                {
                    m_Status = true;
                    GPSEventArgs e = new GPSEventArgs(this.m_Status, this.m_LongitudeDegree, this.m_LongitudeMinute, this.m_LatitudeDegree, this.m_LatitudeMinute, this.m_speed, this.m_bearing, this.m_locationStatus, this.m_SatelliteTotalNums, this.m_satellitesInfoList);
                    if (OnGetGPSInfo != null)
                        OnGetGPSInfo(this, e);
                }
            }
            else
            {
                if (m_Status)
                {
                    m_Status = false;
                    GPSEventArgs e = new GPSEventArgs(this.m_Status, this.m_LongitudeDegree, this.m_LongitudeMinute, this.m_LatitudeDegree, this.m_LatitudeMinute, this.m_speed, this.m_bearing, this.m_locationStatus, this.m_SatelliteTotalNums, this.m_satellitesInfoList);
                    if (OnGetGPSInfo != null)
                        OnGetGPSInfo(this, e);
                }
            }
        }

        private void OnSpeedReceived(byte speed)
        {
            this.m_speed = speed;
        }

        private void OnBearingReceived(short bearing)
        {
            this.m_bearing = bearing;
        }

        private void GetGPSInfo(GPSEventArgs e)
        {
            if (OnGetGPSInfo != null)
                OnGetGPSInfo(this, e);
        }

        private void OnSatelliteTimeChanged(DateTime satelliteTime)
        {
            //logger.Debug("SatelliteTime:{0}", satelliteTime);
            if ((satelliteTime != null && satelliteTime > DateTime.Now.AddMinutes(10)))
            {
                this.SatelliteTime = satelliteTime;
                Helper.SetSystemTime(satelliteTime);
            }
            else
            {
                this.SatelliteTime = DateTime.Now;
            }
        }
    }

    /// <summary>
    /// 卫星信息
    /// </summary>
    public class SatellitesInfo
    {
        #region 伪随机噪声码
        private byte m_PseudoRandomCode = 0;
        /// <summary>
        /// PRN 码(伪随机噪声码)
        /// </summary>
        public byte PseudoRandomCode
        {
            get { return m_PseudoRandomCode; }
            set { m_PseudoRandomCode = value; }
        }
        #endregion

        #region 卫星仰角
        private short m_Azimuth = 0;
        /// <summary>
        /// 卫星仰角
        /// </summary>
        public short Azimuth
        {
            get { return m_Azimuth; }
            set { m_Azimuth = value; }
        }
        #endregion

        #region 卫星方位角
        private byte m_Elevation = 0;
        /// <summary>
        /// 卫星方位角
        /// </summary>
        public byte Elevation
        {
            get { return m_Elevation; }
            set { m_Elevation = value; }
        }
        #endregion

        #region 信噪比
        private byte m_SignalToNoiseRatio = 0;
        /// <summary>
        /// 信噪比
        /// </summary>
        public byte SignalToNoiseRatio
        {
            get { return m_SignalToNoiseRatio; }
            set { m_SignalToNoiseRatio = value; }
        }
        #endregion

        public SatellitesInfo() { }

        public SatellitesInfo(byte pseudoRandomCode, short azimuth, byte elevation, byte signalToNoiseRatio)
        {
            m_PseudoRandomCode = pseudoRandomCode;
            m_Azimuth = azimuth;
            m_Elevation = elevation;
            m_SignalToNoiseRatio = signalToNoiseRatio;
        }
    }
}
