using System;
using System.Globalization;

public class NmeaInterpreter
{
    private NLog.Logger logger = NLog.LogManager.GetLogger("NmeaInterpreter");

	// Represents the zh-CN culture, used for numers in NMEA sentences
	public static CultureInfo NmeaCultureInfo = new CultureInfo("zh-CN");

	// Used to convert knots into miles per hour
	public static double MPHPerKnot = double.Parse("1.150779", NmeaCultureInfo);

	#region Delegates
    public delegate void PositionReceivedEventHandler(string latitudeDegree, string latitudeMinute, string NorS, string longitudeDegree, string longitudeMinute, string EorW);
	public delegate void DateTimeChangedEventHandler(System.DateTime dateTime);
	public delegate void BearingReceivedEventHandler(short bearing);
	public delegate void SpeedReceivedEventHandler(byte speed);
	public delegate void SpeedLimitReachedEventHandler();
	public delegate void FixObtainedEventHandler();
	public delegate void FixLostEventHandler();
    public delegate void SatelliteReceivedEventHandler(byte satelliteTotalNums, byte pseudoRandomCode, short azimuth, byte elevation, byte signalToNoiseRatio);
	public delegate void HDOPReceivedEventHandler(double value);
	public delegate void VDOPReceivedEventHandler(double value);
	public delegate void PDOPReceivedEventHandler(double value);
    public delegate void LocationStatusReceivedEventHandler(byte value);
    public delegate void FixTypeHandler(int e);
	#endregion

	#region Events
	public event PositionReceivedEventHandler PositionReceived = null;
	public event DateTimeChangedEventHandler DateTimeChanged = null;
	public event BearingReceivedEventHandler BearingReceived = null;
	public event SpeedReceivedEventHandler SpeedReceived = null;
	public event SpeedLimitReachedEventHandler SpeedLimitReached = null;
	public event FixObtainedEventHandler FixObtained = null;
	public event FixLostEventHandler FixLost = null;
	public event SatelliteReceivedEventHandler SatelliteReceived = null;
	public event HDOPReceivedEventHandler HDOPReceived = null;
	public event VDOPReceivedEventHandler VDOPReceived = null;
	public event PDOPReceivedEventHandler PDOPReceived = null;
    public event LocationStatusReceivedEventHandler LocationStatusReceived = null;
    public event FixTypeHandler FixType = null;
	#endregion
	
	/// <summary>
    /// Processes information from the GPS receiver
	/// </summary>
	public bool Parse(string sentence)
	{
		// Discard the sentence if its checksum does not match our calculated checksum
		if (!IsValid(sentence)) return false;

        sentence = sentence.Substring(0, sentence.LastIndexOf("*"));
		// Look at the first word to decide where to go next
		switch (GetWords(sentence)[0])
		{
			case "$GPRMC": 
				// A "Recommended Minimum" sentence was found!
				return ParseGPRMC(sentence);
			case "$GPGSV": 
				// A "Satellites in View" sentence was recieved
				return ParseGPGSV(sentence);
			case "$GPGSA":
				return ParseGPGSA(sentence);
            case "$GPGGA":
                return ParseGSGGA(sentence);
            case "$GPVTG":
                return ParseGPVTG(sentence);
			default:
				// Indicate that the sentence was not recognized
				return true;
		}
	}

	/// <summary>
    /// Divides a sentence into individual words
	/// </summary>
	public string[] GetWords(string sentence)
	{
		return sentence.Split(',');
	}

	/// <summary>
    /// Interprets a $GPRMC message
	/// </summary>
	public bool ParseGPRMC(string sentence)
	{
        //try
        //{
            // Divide the sentence into words
            string[] Words = GetWords(sentence);

            // Does the device currently have a satellite fix?
            if (Words[2] != "")
            {
                switch (Words[2])
                {
                    case "A":
                        if (FixObtained != null)
                            FixObtained();
                        break;
                    case "V":
                        if (FixLost != null)
                            FixLost();
                        break;
                }
            }

            // Do we have enough values to describe our location?
            if (Words.Length> 3&& Words[3] != "" & Words[4] != "" & Words[5] != "" & Words[6] != "")
            {
                // Extract latitude and longitude
                //latitude
                string Latitude_Degree = Words[3].Substring(0, 2);
                string Latitude_Minute = Words[3].Substring(2, Words[3].Length - 2);
                //longitude
                string Longitude_Degree = Words[5].Substring(0, 3);
                string Longitude_Minute = Words[5].Substring(3, Words[3].Length - 3);

                // Notify the calling application of the change
                if (PositionReceived != null)
                    PositionReceived(Latitude_Degree, Latitude_Minute, Words[4], Longitude_Degree, Longitude_Minute, Words[6]);
            }
            // Do we have enough values to parse satellite-derived time?
            if (Words[1] != "")
            {
                // Yes. Extract hours, minutes, seconds and milliseconds
                int UtcHours = Convert.ToInt32(Words[1].Substring(0, 2));
                int UtcMinutes = Convert.ToInt32(Words[1].Substring(2, 2));
                int UtcSeconds = Convert.ToInt32(Words[1].Substring(4, 2));
                int UtcMilliseconds = 0;
                // Extract milliseconds if it is available
                if (Words[1].Length > 7)
                {
                    UtcMilliseconds = Convert.ToInt32(Words[1].Substring(7));
                }

                if (Words.Length > 9 && Words[9] != "")
                {
                    try
                    {
                        //logger.Debug("DDMMYY:{0}", Words[9]);
                        int UtcDay = Convert.ToInt32(Words[9].Substring(0, 2));
                        int UtcMonth = Convert.ToInt32(Words[9].Substring(2, 2));
                        int UtcYear = Convert.ToInt32(Words[9].Substring(4, 2));
                        // Now build a DateTime object with all values
                        System.DateTime SatelliteTime = UtcToLocalTime(UtcYear, UtcMonth, UtcDay, UtcHours, UtcMinutes, UtcSeconds);
                        //System.DateTime SatelliteTime = DateTime.ParseExact(dateStr, "yyMMddHHmmss", NmeaCultureInfo);//new System.DateTime(UtcYear, UtcMonth, UtcDay, UtcHours, UtcMinutes, UtcSeconds, UtcMilliseconds);
                        // Notify of the new time, adjusted to the local time zone
                        if (DateTimeChanged != null)
                            DateTimeChanged(SatelliteTime);
                    }
                    catch(Exception ex)
                    {
                        logger.ErrorException("ʱתʧ", ex);
                    }
                }         
            }
            
            //// Do we have enough information to extract the current speed?
            //if (Words[7] != "")
            //{
            //    // Yes.  Parse the speed and convert it to MPH
            //    double Speed = double.Parse(Words[7], NmeaCultureInfo) * MPHPerKnot;
            //    // Notify of the new speed
            //    if(SpeedReceived != null)
            //        SpeedReceived(Speed);
            //    // Are we over the highway speed limit?
            //    if (Speed > 55)
            //        if(SpeedLimitReached != null)
            //            SpeedLimitReached();
            //}
            // Do we have enough information to extract bearing?
            //if (Words[8] != "")
            //{
            //    // Indicate that the sentence was recognized
            //    double Bearing = double.Parse(Words[8], NmeaCultureInfo);
            //    if(BearingReceived != null)
            //        BearingReceived(Bearing);
            //}
        //}
        //catch (Exception ex)
        //{
        //    logger.Info(sentence);
        //    logger.ErrorException("ParseGPRMC.", ex);
        //}
		// Indicate that the sentence was recognized
		return true;
	}

	/// <summary>
    /// Interprets a "Satellites in View" NMEA sentence
	/// </summary>
	public bool ParseGPGSV(string sentence)
	{
        //try
        //{
            byte SatelliteTotalNums = 0;
            byte PseudoRandomCode = 0;
            byte Elevation = 0;
            short Azimuth = 0;
            byte SignalToNoiseRatio = 0;
            // Divide the sentence into words
            string[] Words = GetWords(sentence);
            if (Words.Length < 3)
            {
                return false;
            }
            if (Words[3] != "")
            {
                SatelliteTotalNums = Convert.ToByte(Words[3] == "" ? "0" : Words[3]);
            }

            // Each sentence contains four blocks of satellite information. Read each block and report each satellite's information
            int Count = 0;
            for (Count = 1; Count <= 4; Count++)
            {
                // Does the sentence have enough words to analyze?
                if ((Words.Length - 1) >= (Count * 4 + 3))
                {
                    // Yes. Proceed with analyzing the block.  Does it contain any information?
                    if (Words[Count * 4] != "" & Words[Count * 4 + 1] != "" & Words[Count * 4 + 2] != "" & Words[Count * 4 + 3] != "")
                    {
                        // Yes. Extract satellite information and report it
                        PseudoRandomCode = Convert.ToByte(Words[Count * 4]);
                        Elevation = Convert.ToByte(Words[Count * 4 + 1]);
                        Azimuth = Convert.ToInt16(Words[Count * 4 + 2]);
                        SignalToNoiseRatio = Convert.ToByte(Words[Count * 4 + 3]);

                        // Notify of this satellite's information
                        if (SatelliteReceived != null)
                            SatelliteReceived(SatelliteTotalNums, PseudoRandomCode, Azimuth, Elevation, SignalToNoiseRatio);
                    }
                }
            }
        //}
        //catch (Exception ex)
        //{
        //    logger.Info(sentence);
        //    logger.ErrorException("ParseGPGSV.", ex);
        //}
		// Indicate that the sentence was recognized
		return true;
	}

	/// <summary>
    /// Interprets a "Fixed Satellites and DOP" NMEA sentence
	/// </summary>
	public bool ParseGPGSA(string sentence)
	{
        //try
        //{
            // Divide the sentence into words
            string[] Words = GetWords(sentence);

            if (Words.Length > 2 && Words[15] != "")
            {
                if (FixType != null)
                    FixType(int.Parse(Words[2]));
            }
            // Update the DOP values
            if (Words.Length> 15 && Words[15] != "")
            {
                if (PDOPReceived != null)
                    PDOPReceived(double.Parse(Words[15], NmeaCultureInfo));
            }
            if (Words.Length > 16 && Words[16] != "")
            {
                if (HDOPReceived != null)
                    HDOPReceived(double.Parse(Words[16], NmeaCultureInfo));
            }
            if (Words.Length > 17 && Words[17] != "")
            {
                if (VDOPReceived != null)
                    VDOPReceived(double.Parse(Words[17], NmeaCultureInfo));
            }
        //}
        //catch (Exception ex)
        //{
        //    logger.Info(sentence);
        //    logger.ErrorException("ParseGPGSA.", ex);
        //}
		return true;
	}

    /// <summary>
    /// Interprets a "Global Positioning System Fix Data" NMEA sentence
    /// </summary>
    public bool ParseGSGGA(string sentence)
    {
        byte gpsStatus;
        //try
        //{
            // Divide the sentence into words
            string[] Words = GetWords(sentence);

            if (Words.Length > 6)
            {
                gpsStatus = Words[6].Length == 0 ? (byte)0 : byte.Parse(Words[6]);
                if (LocationStatusReceived != null)
                    LocationStatusReceived(gpsStatus);
            }
        //}
        //catch (Exception ex)
        //{
        //    logger.Info(sentence);
        //    logger.ErrorException("ParseGSGGA.", ex);
        //}
        return true;
    }

    /// <summary>
    /// ٶϢ
    /// </summary>
    public bool ParseGPVTG(string sentence)
    {
        //try
        //{
            // Divide the sentence into words
            string[] Words = GetWords(sentence);

            //logger.Info(sentence);

            if (Words.Length > 7 && Words[7] != "")
            {
                byte Speed = byte.Parse(Words[7].Substring(0, Words[7].LastIndexOf('.')));
                if (SpeedReceived != null)
                    SpeedReceived(Speed);
            }
            if (Words[1] != "")
            {
                short Bearing = short.Parse(Words[1].Substring(0, Words[1].LastIndexOf('.')));
                if (BearingReceived != null)
                    BearingReceived(Bearing);
            }
        //}
        //catch (Exception ex)
        //{
        //    logger.Info(sentence);
        //    logger.ErrorException("ParseGPVTG.", ex);
        //}
        return true;
    }

	/// <summary>
    /// Returns True if a sentence's checksum matches the calculated checksum
	/// </summary>
	public bool IsValid(string sentence)
	{
		// Compare the characters after the asterisk to the calculation
		return sentence.Substring(sentence.IndexOf("*") + 1) == GetChecksum(sentence);
	}

	/// <summary>
    /// Calculates the checksum for a sentence
	/// </summary>
    public string GetChecksum(string sentence)
    {
        // Loop through all chars to get a checksum
        //INSTANT C# NOTE: Commented this declaration since looping variables in 'foreach' loops are declared in the 'foreach' header in C#
        //        char Character = '\0';
        int Checksum = 0;
        foreach (char Character in sentence)
        {
            if (Character == '$')
            {
                // Ignore the dollar sign
            }
            else if (Character == '*')
            {
                // Stop processing before the asterisk
                break;
            }
            else
            {
                // Is this the first value for the checksum?
                if (Checksum == 0)
                {
                    // Yes. Set the checksum to the value
                    Checksum = Convert.ToByte(Character);
                }
                else
                {
                    // No. XOR the checksum with this character's value
                    Checksum = Checksum ^ Convert.ToByte(Character);
                }
            }
        }
        // Return the checksum formatted as a two-character hexadecimal
        return Checksum.ToString("X2");
    }

    public DateTime UtcToLocalTime(int y, int m, int d, int hh, int mm, int ss)
    {
        hh += 8;      // ʱ + 8 Сʱ = ʱ
        if (hh < 24) goto l_out;// ûп죬

        ///////////// ǿļ /////////////////////////////
        hh -= 24;
        d++;        // ڼһ

        switch (m)        // ж
        {
            case 4:
            case 6:                // Сµж
            case 9:
            case 11:
                if (d > 30)
                {
                    d = 1; m++;
                }
                break;
            case 1:
            case 3:                // µж
            case 5:
            case 7:
            case 8:
            case 10:
                if (d > 31)
                {
                    d = 1; m++;
                }
                break;
            case 12:                        // 12 £ҪжǷ

                if (d > 31)
                {
                    y++; d = 1; m = 1;
                }
                break;
            case 2:                            // 2 £ҪжǷ

                if (((y + 2000) % 400 == 0) ||                        // ܱ400һ
                    ((y + 2000) % 4 == 0) && ((y + 2000) % 100 != 0))    // ܱ4ܱ100һ
                {
                    if (d > 29)    // 2£29
                    {
                        m = 3; d = 1;
                    }
                }
                else if (d > 28)    // 2£28
                {
                    m = 3; d = 1;
                }
                break;
        }
        l_out:
        DateTime localTime = new DateTime(y + 2000, m, d, hh, mm, ss);

        return localTime;
    }
}
