﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;


namespace LC
{
    /// <summary>
    /// this class is designed to facilitate bluetooth printer application software development.
    /// </summary>
    public class Printer
    {
        /// <summary>
        /// barocode type
        /// </summary>
        public enum BarcodeType
        {
            /// <summary>
            /// ean
            /// </summary>
            Barcoce_EAN = 1,
            /// <summary>
            /// upc == 12-digit ean
            /// </summary>
            Barcoce_UPC,        
            /// <summary>
            /// isbn numbers (still EAN13)
            /// </summary>
            Barcoce_ISBN,  
            /// <summary>
            /// code 39
            /// </summary>
            Barcoce_39,         
            /// <summary>
            /// code 128 (a,b,c: autoselection)
            /// </summary>
            Barcoce_128,       
            /// <summary>
            ///  code 128 (compact form for digits) 
            /// </summary>
            Barcoce_128C,     
            /// <summary>
            /// code 128, full printable ascii
            /// </summary>
            Barcoce_128B,      
            /// <summary>
            /// interleaved 2 of 5 (only digits)
            /// </summary>
            Barcoce_I25,       
            /// <summary>
            ///  Raw code 128
            /// </summary>
            Barcoce_128RAW,   
            /// <summary>
            /// Codabar
            /// </summary>
            Barcoce_CBR,         
            /// <summary>
            /// MSI
            /// </summary>
            Barcoce_MSI,         
            /// <summary>
            /// Plessey
            /// </summary>
            Barcoce_PLS,         
            /// <summary>
            /// code 93 
            /// </summary>
            Barcoce_93     

        }

        /// <summary>
        /// the status of the printer
        /// </summary>
        public enum PrinterStatus
        {
            /// <summary>
            /// No error
            /// </summary>
            Normal = 0x00,
            /// <summary>
            /// No paper
            /// </summary>
            NoPaper = 0x01,
            /// <summary>
            /// LF key down.
            /// </summary>
            LFKeyDown = 0x02,
            /// <summary>
            /// Low power
            /// </summary>
            LowPower =0x04
        }


        /// <summary>
        /// This function initializes the bluetooth printer
        /// </summary>
        /// <param name="printer">Handle to the printer to be connected. The value of printer can come from CreateFile function.</param>
        /// <returns>true indicates success. false indicates failure.</returns>
        [DllImport("Printer.dll", EntryPoint = "prn_Init")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool Init(IntPtr printer);

        /// <summary>
        /// This function uninitializes the bluetooth printer.
        /// </summary>
        [DllImport("Printer.dll", EntryPoint = "prn_Deinit")]
        public static extern void Deinit();

        /// <summary>
        /// This function setup a page with the specified width, height.
        /// </summary>
        /// <param name="pagewidth">Specifies the page width, in pixels.</param>
        /// <param name="pageheight">Specifies the page height, in pixels.</param>
        /// <returns>true indicates success. false indicates failure.</returns>
        /// <remarks>1 pixel is eques to 0.125mm.</remarks>
        [DllImport("Printer.dll", EntryPoint = "prn_PageSetup")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool PageSetup(int pagewidth, int pageheight);

        /// <summary>
        /// This function print the page.
        /// </summary>
        /// <param name="rotate">true indicates 90 degree clockwise rotation</param>
        /// <returns>the status of the printer</returns>
        [DllImport("Printer.dll", EntryPoint = "prn_PagePrint")]
        public static extern int PagePrint([MarshalAs(UnmanagedType.Bool)]bool rotate);

        /// <summary>
        /// This function clear the page.
        /// </summary>
        [DllImport("Printer.dll", EntryPoint = "prn_PageClear")]
        public static extern void PageClear();

        /// <summary>
        /// Get the device context of the page
        /// </summary>
        /// <returns>handle to device context</returns>
        [DllImport("Printer.dll", EntryPoint = "prn_GetPageCanvas")]
        public static extern IntPtr GetPageCanvas();

        /// <summary>
        /// This function draw a line that connects two points.
        /// </summary>
        /// <param name="x0">The x coordinate where the line starts. </param>
        /// <param name="y0">The y coordinate where the line starts. </param>
        /// <param name="x1">The x coordinate where the line ends. </param>
        /// <param name="y1">The y coordinate where the line ends. </param>
        /// <param name="linewidth">the width of the line, in pixel</param>
        /// <returns>true indicates success. false indicates failure.</returns>
        /// <remarks>1 pixel is eques to 0.125mm.</remarks>
        [DllImport("Printer.dll", EntryPoint = "prn_DrawLine")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool DrawLine(int x0, int y0, int x1, int y1, int linewidth);

        /// <summary>
        /// This function draw the formatted text with specified parameters
        /// </summary>
        /// <param name="x">the x coordinate of the upper-left corner of the text</param>
        /// <param name="y">the y coordinate of the upper-left corner of the text</param>
        /// <param name="text">the string that contains the text to draw</param>
        /// <param name="fontname">the string that specifies the typeface name of the font</param>
        /// <param name="fontsize">the size of the font, in pixel</param>
        /// <param name="rotate">0 indicates no rotate; 1 indicates 90 degree clockwise rotation ; 2 indicates 180 degree clockwise rotation; 3 indicates 270 degree clockwise rotation;</param>
        /// <param name="bold">the weight of the font</param>
        /// <param name="underline">specifies an underlined font</param>
        /// <param name="strikeout">specifies a strikeout font </param>
        /// <param name="italic">specifies an italic font </param>
        /// <param name="reverse">reverse the backcorlr and forecolor</param>
        /// <returns>true indicates success. false indicates failure.</returns>
        /// <remarks>1 pixel is eques to 0.125mm.</remarks>
        [DllImport("Printer.dll", EntryPoint = "prn_DrawText")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool DrawText([MarshalAs(UnmanagedType.LPWStr)]string text, int x, int y, [MarshalAs(UnmanagedType.LPWStr)]string fontname, int fontsize, int rotate, [MarshalAs(UnmanagedType.Bool)]bool bold, [MarshalAs(UnmanagedType.Bool)]bool underline, [MarshalAs(UnmanagedType.Bool)]bool strikeout, [MarshalAs(UnmanagedType.Bool)]bool italic, [MarshalAs(UnmanagedType.Bool)]bool reverse);

        /// <summary>
        /// This function draw the formatted text with specified parameters
        /// </summary>
        /// <param name="x">the x coordinate of the upper-left corner of the barcode</param>
        /// <param name="y">the y coordinate of the upper-left corner of the barcode</param>
        /// <param name="text">the string that contains the text of the barcode</param>
        /// <param name="barcodetype">the type of the barcode</param>
        /// <param name="rotate">0 indicates no rotate; 1 indicates 90 degree clockwise rotation ; 2 indicates 180 degree clockwise rotation; 3 indicates 270 degree clockwise rotation;</param>
        /// <param name="linewidth">the width of the line, in pixel</param>
        /// <param name="height">the hegith of the barcode</param>
        /// <returns>true indicates success. false indicates failure.</returns>
        /// <remarks>1 pixel is eques to 0.125mm.</remarks>
        [DllImport("Printer.dll", EntryPoint = "prn_DrawBarcode")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool DrawBarcode([MarshalAs(UnmanagedType.LPWStr)]string text, int x, int y, int barcodetype, int rotate, int linewidth, int height);


        /// <summary>
        /// This function draw the specified bitmap from a source rectangle into a destination rectangle
        /// </summary>
        /// <param name="bmp">the handle to the specified bitmap</param>
        /// <param name="xDest">Specifies the x-coordinate, in pixel, of the upper left corner of the destination rectangle</param>
        /// <param name="yDest">Specifies the y-coordinate, in pixel, of the upper left corner of the destination rectangle</param>
        /// <param name="widthDest">Specifies the width, in pixel, of the destination rectangle.</param>
        /// <param name="heightDest">Specifies the height, in pixel, of the destination rectangle</param>
        /// <param name="xSrc">Specifies the x-coordinate, in pixel, of the upper left corner of the source rectangle</param>
        /// <param name="ySrc">Specifies the y-coordinate, in pixel, of the upper left corner of the source rectangle</param>
        /// <param name="widthSrc">Specifies the width, in pixel, of the source rectangle</param>
        /// <param name="heightSrc">Specifies the height, in pixel, of the source rectangle</param>
        /// <param name="backColor">Indicates the color in the bitmap that will be rendered as background.</param>
        /// <returns>true indicates success. false indicates failure.</returns>
        /// <remarks>1 pixel is eques to 0.125mm.</remarks>
        [DllImport("Printer.dll", EntryPoint = "prn_DrawBitmap")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool DrawBitmap(IntPtr bmp, int xDest, int yDest, int widthDest, int heightDest, int xSrc, int ySrc, int widthSrc, int heightSrc, uint backColor);

        /// <summary>
        /// This function query the status of the printer
        /// </summary>
        /// <returns>the status of the printer</returns>
        [DllImport("Printer.dll", EntryPoint = "prn_GetPrinterStatus")]
        public static extern int GetPrinterStatus();

    }
}
